local LCCC = LibCodesCommonCode
local LEJ = LibExtendedJournal
local RCR = Raidificator


--------------------------------------------------------------------------------
-- Extended Journal
--------------------------------------------------------------------------------

local TAB_NAME = "Raidificator"
local FRAME = RaidificatorFrame
local DATA_TYPE = 1
local SORT_TYPE = 1

local Initialized = false
local Dirtiness = 0

function RCR.InitializeHistory( )
	LEJ.RegisterTab(TAB_NAME, {
		name = SI_RCR_HISTORY,
		order = 500,
		iconPrefix = "/esoui/art/journal/journal_tabicon_leaderboard_",
		control = FRAME,
		settingsPanel = RCR.settingsPanel,
		binding = "RCR_HISTORY",
		slashCommands = { "/raidificator", "/rcr" },
		callbackShow = function( )
			RCR.LazyInitializeHistory()
			RCR.RefreshHistory(true)
		end,
	})

	LCCC.RegisterLinkHandler("rcr", function() LEJ.Show(TAB_NAME) end)
end

function RCR.LazyInitializeHistory( )
	if (not Initialized) then
		Initialized = true

		-- Refresh character names, in case any were renamed
		for i = 1, GetNumCharacters() do
			local name, _, _, classId, _, _, id = GetCharacterInfo(i)
			if (RCR.GetCharacterData(id)) then
				RCR.UpdateCharacterData(id, zo_strformat(SI_PLAYER_NAME, name), classId)
			end
		end

		RCR.list = RaidificatorList:New(FRAME)
	end
end

function RCR.RefreshHistory( noActiveCheck )
	if (Initialized and Dirtiness > 0 and (noActiveCheck or LEJ.IsTabActive(TAB_NAME))) then
		RCR.list:RefreshData()
		Dirtiness = 0 -- Redundant fail-safe; this should already be set to 0 by the refresh
	end
end

function RCR.NewClearAdded( contentId )
	RCR.vars.contentId = contentId
	if (Initialized) then
		Dirtiness = 1
		RCR.RefreshHistory()
	end
end


--------------------------------------------------------------------------------
-- RaidificatorList
--------------------------------------------------------------------------------

RaidificatorList = ExtendedJournalSortFilterList:Subclass()
local RaidificatorList = RaidificatorList

function RaidificatorList:Setup( )
	ZO_ScrollList_AddDataType(self.list, DATA_TYPE, "RaidificatorListRow", 30, function(...) self:SetupItemRow(...) end)
	ZO_ScrollList_EnableHighlight(self.list, "ZO_ThinListHighlight")
	self:SetAlternateRowBackgrounds(true)

	self.masterList = { }

	local sortKeys = {
		["server"]    = { tiebreaker = "timestamp", tieBreakerSortOrder = ZO_SORT_ORDER_DOWN },
		["timestamp"] = { isNumeric = true },
		["character"] = { caseInsensitive = true, tiebreaker = "timestamp", tieBreakerSortOrder = ZO_SORT_ORDER_DOWN },
		["score"]     = { sNumeric = true, tiebreaker = "timestamp", tieBreakerSortOrder = ZO_SORT_ORDER_DOWN },
		["duration"]  = { sNumeric = true, tiebreaker = "timestamp", tieBreakerSortOrder = ZO_SORT_ORDER_DOWN },
		["extra"]     = { tiebreaker = "timestamp", tieBreakerSortOrder = ZO_SORT_ORDER_DOWN },
	}

	self.currentSortKey = "timestamp"
	self.currentSortOrder = ZO_SORT_ORDER_DOWN
	self.sortHeaderGroup:SelectAndResetSortForKey(self.currentSortKey)
	self.sortFunction = function( listEntry1, listEntry2 )
		return ZO_TableOrderingFunction(listEntry1.data, listEntry2.data, self.currentSortKey, sortKeys, self.currentSortOrder)
	end

	self.contentIdToIndex = { }
	local contentList = { }
	for _, content in ipairs(RCR.SCORED_CONTENT) do
		local name = LCCC.GetZoneName(content[2])
		if (name ~= "") then
			table.insert(contentList, { name, content[1] })
			self.contentIdToIndex[content[1]] = #contentList
		end
	end

	self.filterDrop = ZO_ComboBox_ObjectFromContainer(self.frame:GetNamedChild("FilterDrop"))
	self:InitializeComboBox(self.filterDrop, { list = contentList, key = 1, dataKey = 2 }, self.contentIdToIndex[RCR.vars.contentId])

	self.accounts = { }
	self.accountsInitialized = false
	local control = self.frame:GetNamedChild("AccountDrop")
	control:GetNamedChild("Caption"):SetText(GetString(SI_LEJ_ACCOUNT))
	self.accountDrop = ZO_ComboBox_ObjectFromContainer(control)

	self:RefreshData()
end

function RaidificatorList:BuildMasterList( )
	self.masterList = { }

	for _, entry in ipairs(RCR.history.clears) do
		local contentId, server, timestamp, character, score, duration, extra, group = zo_strsplit(",", entry)
		local userId, charName, classId = RCR.GetCharacterData(character)
		if (charName) then
			extra = { zo_strsplit(";", extra) }
			table.insert(self.masterList, {
				type = SORT_TYPE,
				contentId = contentId,
				server = server,
				timestamp = tonumber(timestamp),
				userId = userId,
				character = charName,
				classIndex = GetClassIndexById(classId),
				score = tonumber(score),
				duration = tonumber(duration),
				extra = string.format((#extra < 3) and "%02d%02d" or "%03d%d%d", unpack(extra)), -- The text here is used only for the sort algorithm and is padded with leading zeros for that
				extraRaw = extra,
				group = group,
			})
			if (not self.accountsInitialized) then
				self.accounts[userId] = true
			end
		end
	end

	if (not self.accountsInitialized) then
		self.accountsInitialized = true
		self:InitializeAccounts()
	end
end

function RaidificatorList:FilterScrollList( )
	local scrollData = ZO_ScrollList_GetDataList(self.list)
	ZO_ClearNumericallyIndexedTable(scrollData)

	if (Dirtiness > 0) then
		-- New clear added to history
		LEJ.SelectComboBoxItemByIndex(self.filterDrop, self.contentIdToIndex[RCR.vars.contentId], true)
		Dirtiness = 0
	else
		RCR.vars.contentId = self.filterDrop:GetSelectedItemData().data
	end

	local selectedAccount = self.accountDrop:GetSelectedItemData()
	local userId = selectedAccount.name
	local accountIndex = selectedAccount.id
	RCR.vars.allAccounts = accountIndex == 1

	for _, data in ipairs(self.masterList) do
		if (RCR.vars.contentId == data.contentId and (accountIndex == 1 or userId == data.userId)) then
			table.insert(scrollData, ZO_ScrollList_CreateDataEntry(DATA_TYPE, data))
		end
	end

	self.frame:GetNamedChild("Counter"):SetText(string.format("%d", #scrollData))
end

function RaidificatorList:SetupItemRow( control, data )
	local cell

	cell = control:GetNamedChild("Server")
	cell:SetTexture(RCR.GetTexture(data.server))

	cell = control:GetNamedChild("Timestamp")
	cell.normalColor = ZO_DEFAULT_TEXT
	cell:SetText(os.date("%Y-%m-%d %H:%M", data.timestamp))

	cell = control:GetNamedChild("Class")
	cell:SetTexture(data.classIndex and select(8, GetClassInfo(data.classIndex)) or RCR.GetTexture())

	cell = control:GetNamedChild("Character")
	cell.normalColor = ZO_DEFAULT_TEXT
	cell:SetText(data.character)

	cell = control:GetNamedChild("Score")
	cell.normalColor = ZO_DEFAULT_TEXT
	cell:SetText(data.score)

	cell = control:GetNamedChild("Duration")
	cell.normalColor = ZO_DEFAULT_TEXT
	cell:SetText(RCR.FormatTime(data.duration, 2))

	cell = control:GetNamedChild("Extra")
	cell.normalColor = ZO_DEFAULT_TEXT
	cell:SetText(RCR.FormatExtra(unpack(data.extraRaw)))

	cell = control:GetNamedChild("ExtraIcon")
	cell:SetTexture((#data.extraRaw < 3) and "/esoui/art/trials/vitalitydepletion.dds" or "/esoui/art/endlessdungeon/thick_outline_icon_progression_arc.dds")

	self:SetupRow(control, data)
end

function RaidificatorList:InitializeAccounts( )
	local userId = GetDisplayName()
	self.accounts[userId] = true
	local accountNames = LCCC.GetSortedKeys(self.accounts, userId)
	table.insert(accountNames, 1, GetString(SI_RCR_ALL_ACCOUNTS))
	self:InitializeComboBox(self.accountDrop, { list = accountNames }, RCR.vars.allAccounts and 1 or 2)
end


--------------------------------------------------------------------------------
-- XML Handlers
--------------------------------------------------------------------------------

local Tooltip = InformationTooltip

function RaidificatorListRow_OnMouseEnter( control )
	local data = ZO_ScrollList_GetData(control)
	RCR.list:Row_OnMouseEnter(control)

	if (data.group and data.group ~= "") then
		local iconSize = 22
		local atSize = control:GetNamedChild("Character"):GetStringWidth("@") / GetUIGlobalScale()

		local results = { }
		local members = { zo_strsplit(";", data.group) }

		for _, member in ipairs(members) do
			local charName, userId, classId = zo_strsplit(":", member)
			if (classId) then
				table.insert(results, zo_iconTextFormat(select(8, GetClassInfo(GetClassIndexById(classId))), iconSize, iconSize, DecorateDisplayName(userId)))
				table.insert(results, zo_iconTextFormat(RCR.GetTexture(), iconSize + atSize, iconSize, charName))
			else
				table.insert(results, charName)
			end
		end

		Tooltip = LEJ.InitializeTooltip(Tooltip)
		Tooltip:AddLine(GetString(SI_RCR_GROUP_MEMBERS))
		SetTooltipText(Tooltip, table.concat(results, "\n"))
	end
end

function RaidificatorListRow_OnMouseExit( control )
	RCR.list:Row_OnMouseExit(control)

	ClearTooltip(Tooltip)
end
